/**
 * Copyright (c) 2012 - 2017 Data In Motion and others.
 * All rights reserved. 
 * 
 * This program and the accompanying materials are made available under the terms of the 
 * Eclipse Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Data In Motion - initial API and implementation
 */
package org.eclipse.ece.messaging.mqtt;

import java.util.Formatter;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ThreadLocalRandom;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.MqttPersistenceException;

/**
 * Simple MQTT client, that sends simple messages
 * @author Mark Hoffmann
 * @since 14.10.2017
 */
public class MQTTClient {

	private final Logger logger = Logger.getLogger("mqttClient");
	private final Timer timer;
	private final String url;
	private final String topic;
	private final String name;
	private final int minValue;
	private final int maxValue;
	private final String messageTemplate;
	private MqttClient mqttClient;
	private boolean running = false;
	private TimerTask task = new TimerTask() {

		/* 
		 * (non-Javadoc)
		 * @see java.util.TimerTask#run()
		 */
		@Override
		public void run() {
				long ts = System.currentTimeMillis();
				int value = ThreadLocalRandom.current().nextInt(minValue, maxValue + 1);
				Formatter formatter = new Formatter();
				String content = formatter.format(messageTemplate, value, name, ts).toString();
				formatter.close();
				MqttMessage msg = new MqttMessage(content.getBytes());
				try {
					synchronized (mqttClient) {
						mqttClient.publish(topic, msg);
					}
				} catch (MqttPersistenceException e) {
					logger.log(Level.SEVERE, "Persistence error on publish for client " + name, e);
				} catch (MqttException e) {
					logger.log(Level.SEVERE, "Mqtt error on publish for client " + name, e);
				}
		}
	};

	public MQTTClient(String name, String url, String topic, String messageTemplate, int minValue, int maxValue) {
		this.name = name;
		this.url = url;
		this.topic = topic;
		this.messageTemplate = messageTemplate;
		this.minValue = minValue;
		this.maxValue = maxValue;
		this.timer = new Timer("timer-" + name);
	}

	/**
	 * Returns <code>true</code>, if the client is running
	 * @return <code>true</code>, if the client is running
	 */
	public boolean isRunning() {
		return running;
	}

	/**
	 * Starts the client
	 */
	public void startClient() {
		try {
			mqttClient = new MqttClient(url, name);
			mqttClient.connect();
			connectClient(topic);
			running = true;
		} catch (MqttException e) {
			logger.log(Level.SEVERE, "Error starting Mqtt client  " + name, e);
		}
	}

	public void stopClient() {
		try {
			mqttClient.disconnect();
			mqttClient.close();
		} catch (MqttException e) {
			logger.log(Level.SEVERE, "Error stopping Mqtt client  " + name, e);
		}
		running = false;
	}


	/**
	 * Connects the check client to
	 * @param topic the topic to connect
	 * @throws MqttException
	 */
	private void connectClient(String topic) throws MqttException {
		mqttClient.subscribe(topic);
		mqttClient.setCallback(new MqttCallback() {

			@Override
			public void messageArrived(String arg0, MqttMessage arg1) throws Exception {
				String v = new String(arg1.getPayload());
				if (v.startsWith("start") && v.endsWith(name)) {
					logger.info("Starting timer task for cient " + name);
					timer.schedule(task, 10l, 1000l);
				}
				if (v.startsWith("stop") && v.endsWith(name)) {
					logger.info("Stopping timer task for cient " + name);
					timer.cancel();
				}

			}

			@Override
			public void deliveryComplete(IMqttDeliveryToken arg0) {
			}

			@Override
			public void connectionLost(Throwable arg0) {
			}
		});
	}

}
